#!/usr/bin/env python

# Using the late Oregonian's pumpkin gradient

# author: Pocholo
# date: 9/5/20

# Comments directed to http://gimpchat.com

# Installation:
# This script should be placed in the user plugins folder

# Copyright (C)

#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by   
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY# without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <https://www.gnu.org/licenses/>.   


from gimpfu import *

def pm_halloween_happy_pumpkin(size, BgColor):
		
	#Variables
	x = size/500
		
	#Create background layer
	img = pdb.gimp_image_new(size, size, RGB)
	backLayer = pdb.gimp_layer_new(img, size, size, RGB, "Background", 100, LAYER_MODE_NORMAL)
	pdb.gimp_image_add_layer(img, backLayer, 0)
	pdb.gimp_context_set_foreground(BgColor)
	pdb.gimp_drawable_fill(backLayer, FILL_FOREGROUND)
	
	pdb.gimp_display_new(img)
	
	#Create the pumpkin layer 
	pumLayer = pdb.gimp_layer_new(img, size, size, RGB, "Pumpkin", 100, LAYER_MODE_NORMAL)
	pdb.gimp_image_add_layer(img, pumLayer, 0)
	pdb.gimp_layer_add_alpha(pumLayer)
	pdb.gimp_drawable_fill(pumLayer, FILL_TRANSPARENT)
	
	#Using a selected gradient to create the grooves of the pumpkin
	pdb.gimp_context_set_gradient("Pumpkin")
	pdb.gimp_context_set_gradient_repeat_mode(1)
	pdb.gimp_drawable_edit_gradient_fill(pumLayer, 0, 0, TRUE, 3, 0.20, TRUE, 0*x, 250*x, 20*x, 250*x)
		
	#using map to create the pumpkin shape
	pdb.plug_in_map_object(img, pumLayer, 
                                       1,    # Type of mapping (0=plane,1=sphere,2=box,3=cylinder)
                           0.5, 0.5, 2.0,    # viewpoint
                           0.5, 0.5, 0.0,    # object pos
                           1.0, 0.0, 0.0,    # first axis
                           0.0, 1.0, 0.0,    # 2nd axis
                          0.0, 0.0, 20.0,    # axis rotation
                                       0,    # Type of lightsource (0=point,1=directional,2=none)
                         (255, 255, 255),    # Lightsource color (r,g,b)
                           0.5, 0.1, 2.0,    # light position
                         -1.0, -1.0, 1.0,    # light direction
             0.80, 0.70, 0.20, 0.0, 27.0,    # material (amb, diff, refl, spec, high)
                                    True,    # antialias
                                   False,    # tile
                                   False,    # new image
                                    True,    # transparency
                                    0.25,    # Sphere/cylinder radius (only used when maptype=1 or 3)
                           0.5, 0.5, 0.5,    # box size
                                    1.00,    # unused parameter Cylinder length
                                    None,    # Box front face (set these to -1 if not used)
                                    None,    # Box back face
                                    None,    # Box top face									
                                    None,    # Box bottom face
                                    None,    # Box left face
                                    None,    # Box right face
                                    None,    # Cylinder top face (unused parameter)
                                    None)    # Cylinder bottom face (unused parameter)
	
	#Use g'mic spherize filter
	pdb.plug_in_gmic_qt(img, pumLayer, 1, 0, "-v - -fx_spherize 65,0.6,0,50,50,0.25,0,0,0") 
	pdb.gimp_image_select_item(img, 2, pumLayer)
	pdb.gimp_selection_shrink(img, 2)
	pdb.gimp_selection_feather(img, 7)						
	pdb.gimp_selection_invert(img)
	pdb.gimp_edit_clear(pumLayer)
	pdb.gimp_edit_clear(pumLayer)
	pdb.gimp_selection_none(img)
	pdb.gimp_levels(pumLayer, 
					        4,    #HISTOGRAM-ALPHA
						  200,    #low_input 
						  255,    #high_input 						 
						  1.5,    #gamma 
						    0,    #low_output 
						  255)    #high_output 
						  	
	#The skin fiber
	pdb.gimp_image_select_item(img, 2, pumLayer)
	fiberLayer = pdb.gimp_layer_new(img, size, size, RGB, "Fiber", 100, LAYER_MODE_OVERLAY)
	pdb.gimp_image_add_layer(img, fiberLayer, 0)
	pdb.gimp_layer_add_alpha(fiberLayer)
	pdb.gimp_drawable_fill(fiberLayer, FILL_TRANSPARENT)		
	pdb.gimp_context_set_pattern("Fibers")
	pdb.gimp_edit_bucket_fill(fiberLayer, BUCKET_FILL_PATTERN, LAYER_MODE_NORMAL, 100, 255, FALSE, 0, 0)
	pdb.plug_in_mblur(img, fiberLayer, 0, 15*x, 90, 250*x, 250*x)
	pdb.plug_in_mblur(img, fiberLayer, 2, 10*x, 0, 250*x, 250*x)
	pdb.gimp_selection_none(img)
	
	#The pumpkin stem
	stemLayer = pdb.gimp_layer_new(img, size, size, RGB, "Crease", 100, LAYER_MODE_NORMAL)
	pdb.gimp_image_add_layer(img, stemLayer, 2)
	pdb.gimp_layer_add_alpha(stemLayer)
	pdb.gimp_drawable_fill(stemLayer, FILL_TRANSPARENT)
	pdb.gimp_brushes_set_brush("Stem")
	pdb.gimp_context_set_brush_size(150*x)
	pdb.gimp_context_set_brush_aspect_ratio(0)
	pdb.gimp_context_set_brush_angle(0)
	pdb.gimp_context_set_brush_spacing(0.01)
	pdb.gimp_context_set_brush_default_hardness()
	pdb.gimp_context_set_paint_method("gimp-paintbrush")
	pdb.gimp_paintbrush_default(stemLayer, 2, [260*x, 130*x])
			
	#Create the layer for the pumpkin face
	faceLayer = pdb.gimp_layer_new(img, size, size, RGB, "Pumpkin face", 100, LAYER_MODE_NORMAL)
	pdb.gimp_image_add_layer(img, faceLayer, 0)
	pdb.gimp_layer_add_alpha(faceLayer)
	pdb.gimp_drawable_fill(faceLayer, FILL_TRANSPARENT)	
		
	#Create the brush for pumpkin eyes and nose
	actual_name = pdb.gimp_brush_new("Triangle hardness 100")
	shape_out = pdb.gimp_brush_set_shape("Triangle hardness 100", 2)     				#BRUSH-GENERATED-DIAMOND
	radius_out = pdb.gimp_brush_set_radius("Triangle hardness 100", 46*x)  				#radius_in
	spikes_out = pdb.gimp_brush_set_spikes("Triangle hardness 100", 3)   				#spikes_in
	hardness_out = pdb.gimp_brush_set_hardness("Triangle hardness 100", 1.0*x)			#hardness_in
	aspect_ratio_out = pdb.gimp_brush_set_aspect_ratio("Triangle hardness 100", 1.5)    #aspect_ratio_in
	angle_out = pdb.gimp_brush_set_angle("Triangle hardness 100", 150)					#angle_in
	pdb.gimp_brush_set_spacing("Triangle hardness 100", 10*x)								#spacing
	pdb.gimp_context_set_brush("Triangle hardness 100")
	
	#Paint the eyes
	pdb.gimp_context_set_brush_size(90*x)
	pdb.gimp_context_set_foreground("Black")
	pdb.gimp_paintbrush_default(faceLayer, 2, [195*x, 230*x])
	pdb.gimp_paintbrush_default(faceLayer, 2, [305*x, 230*x])
	
	#Paint the nose
	pdb.gimp_context_set_brush_size(50)
	pdb.gimp_paintbrush_default(faceLayer, 2, [250*x, 270*x])
	
	#Delete the brush
	pdb.gimp_brush_delete("Triangle hardness 100")
	
	#Create the layer for the pumpkin mouth
	mouthLayer = pdb.gimp_layer_new(img, size, size, RGB, "Pumpkin mouth", 100, LAYER_MODE_NORMAL)
	pdb.gimp_image_add_layer(img, mouthLayer, 0)
	pdb.gimp_layer_add_alpha(mouthLayer)
	pdb.gimp_drawable_fill(mouthLayer, FILL_TRANSPARENT)	
	
	#Create the mouth
	pdb.gimp_image_select_rectangle(img, 2, 204*x, 280*x, 90*x, 45*x)
	pdb.gimp_context_set_foreground("Black")
	pdb.gimp_edit_fill(mouthLayer, FILL_FOREGROUND)
	pdb.gimp_selection_none(img)
	pdb.plug_in_gmic_qt(img, mouthLayer, 1, 0, "-v - -fx_spherize 50,5.5,0,50,50,0,0,0,0") 
	pdb.gimp_layer_translate(mouthLayer, 0*x, -75*x)
	pdb.gimp_layer_resize_to_image_size(mouthLayer)
	pdb.gimp_layer_resize_to_image_size(mouthLayer)
	faceLayer = pdb.gimp_image_merge_down(img, mouthLayer, CLIP_TO_IMAGE)
	pdb.gimp_image_select_item(img, 2, faceLayer)
	pdb.gimp_context_set_foreground((255, 134, 22))
	pdb.gimp_edit_bucket_fill(faceLayer, BUCKET_FILL_FG, LAYER_MODE_NORMAL, 100, 255, FALSE, 0, 0)	
	
	#Drop Shadow
	pdb.python_layerfx_drop_shadow(img, faceLayer,
									   (74, 65, 0),   #color
											   100,   #opacity
												 0,   #contou - linear
												 0,   #noise
												16,   #mode - grain merge
												80,   #spread
											  20*x,   #size
											   120,   #offset_angle
											     1,   #offset_distance
											  TRUE,   #knockout
											 FALSE)   #merge
											 
	#Inner Shadow
	pdb.python_layerfx_inner_shadow(img, faceLayer, 
								     (0, 0, 0),   #color
									       100,   #opacity
										     0,   #contou - linear
										     0,   #noise
										    21,   #mode - grain merge
										     1,   #source
										    35,   #choke 
									        60,   #size 
										   -90,   #offset_angle 
										     1,   #offset_distance 
									     FALSE)  #merge									 
		
	#Inner glow
	dropshadow = pdb.gimp_image_get_layer_by_name(img, "Pumpkin face-dropshadow")
	pdb.gimp_image_set_active_layer(img, dropshadow)
	pdb.python_layerfx_inner_glow(img, dropshadow, 
								(235, 136, 0),   #color
										   75,   #opacity
											0,   #contou
											0,   #noise
											3,   #mode - multiply 
											1,   #source
											0,   #choke 
										 10*x,   #size 
										 FALSE)   #merge
	
	#Make the inside of the pumkin
	inLayer = pdb.gimp_layer_new(img, size, size, RGB, "Inner pumpkin", 100, LAYER_MODE_OVERLAY)
	pdb.gimp_image_add_layer(img, inLayer, 0)
	pdb.gimp_layer_add_alpha(inLayer)
	pdb.gimp_drawable_fill(inLayer, FILL_TRANSPARENT)		
	pdb.gimp_context_set_pattern("Stone")
	pdb.gimp_edit_bucket_fill(inLayer, BUCKET_FILL_PATTERN, LAYER_MODE_NORMAL, 100, 255, FALSE, 0, 0)
	pdb.plug_in_mblur(img, inLayer, 0, 8*x, 90, 250*x, 250*x)
	pdb.plug_in_mblur(img, inLayer, 2, 20*x, 0, 250*x, 250*x)
	pdb.gimp_selection_none(img)
	
	#Make a new from visible and rename the layer	
	pdb.gimp_item_set_visible(backLayer, FALSE)
	Pumpkin = pdb.gimp_layer_new_from_visible(img, img, "Halloween Pumpkin")
	pdb.gimp_image_add_layer(img, Pumpkin, 0)
	
	#Make the other layer invisible
	pdb.gimp_item_set_visible(pumLayer, FALSE)
	pdb.gimp_item_set_visible(stemLayer, FALSE)
	pdb.gimp_item_set_visible(dropshadow, FALSE)
	innerglow = pdb.gimp_image_get_layer_by_name(img, "Pumpkin face-dropshadow-innerglow")
	pdb.gimp_item_set_visible(innerglow, FALSE)
	pdb.gimp_item_set_visible(faceLayer, FALSE)
	innershadow = pdb.gimp_image_get_layer_by_name(img, "Pumpkin face-innershadow")
	pdb.gimp_item_set_visible(innershadow, FALSE)
	pdb.gimp_item_set_visible(inLayer, FALSE)
				
register(
	"pm_halloween_happy_pumpkin",
	"Creates a carved Halloween pumpkin",
	"Creates a carved Halloween pumpkin",
	"Pocholo",
	"Pocholo",
	"2020",
	"Halloween happy pumpkin",
	"",
[	
	(PF_SPINNER, "size", "Pumpkin size", 500, (1, 1000, 1)),
	(PF_COLOR, "Bgcolor", "Background color", (255, 255, 255)),
],
[],
pm_halloween_happy_pumpkin, menu="<Image>/Pocholo-scripts/Halloween happy pumpkin")
main()		